# The information in this publication is provided "as is." Dell Inc. makes no representations or warranties of any kind with respect to the information in this publication, and specifically disclaims implied warranties of merchantability or fitness for a particular purpose.
# Use, copying, and distribution of any software described in this publication requires an applicable software license.
# Copyright (c) 2020 - v1.0 Dell Inc. or its subsidiaries. All Rights Reserved. Dell Technologies, Dell, EMC, Dell EMC and other trademarks are trademarks of Dell Inc. or its subsidiaries. Other trademarks may be trademarks of their respective owners.
import os
import platform
from struct import *
import json
import copy
import sys
from collections import OrderedDict


# global SMBIOS Raw
SMBiosRawDataObj = None

# global SMBIOS Access
SMBiosAccessObj = None

# global Collect Data
CollectDataObj = None




# ################################### #
# Class to fetch the smbios details
# Provides SMBIOS formatted data
# ################################### #
class SMBiosRawData:
    def __init__(self, filename):
        global SMBiosRawDataObj
        if SMBiosRawDataObj:
            self.refObj = SMBiosRawDataObj
            return
        SMBiosRawDataObj = self
        self.refObj = None
        self.tableNumberGen = None
        self.filename = filename
        self.thePlatform = platform.system()
        self.exitCode = 1  # failed
        majorVersion = '??'
        minorVersion = '??'
        if self.filename:
            anInput = open(self.filename, 'r')
            if not anInput:
                raise "Can't open %s" % self.filename
            rawBuffer = ''
            while True:
                rawIn = anInput.read()
                if not rawIn:
                    break
                rawBuffer += rawIn
            anInput.close()
        else:
            if self.thePlatform == 'Linux':
                smbDumpFile = 'dmidecodedumpfile'
                exitStatus = os.system('dmidecode --dump-bin ' + smbDumpFile + ' > /dev/null')
                self.exitCode = os.WEXITSTATUS(exitStatus)
                anInput = open(smbDumpFile, 'r')
                rawData = anInput.read()
                anInput.close()
                if os.path.exists(smbDumpFile):
                    os.remove(smbDumpFile)

                (smrev, unused) = unpack('4s27s', rawData[:31])
                if smrev == "_SM_":
                    (anchorString, checkSum, entryPointLength, majorVersion, minorVersion, \
                    maxStructSize, entryRevision, fa0, fa1, fa2, \
                    fa3, fa4, intAnchor, intCheckSum, tableSize, \
                    tableAddress, numStructs, bcdRevision) = unpack('4scccchc5c5schihc', rawData[:31])
                elif smrev == "_SM3":
                    (anchorString, checkSum, entryPointLength, majorVersion, minorVersion, docRev, \
                    entryRevision, reservedsm3, tableSize, tableAddress) = unpack('5sccccccciq', rawData[:24])
                else:
                    expsm = "Unknown sm version encountered"
                    print expsm
                    quit()

                rawBuffer = '\nNode,Active,DmiRevision,InstanceName,Size,SMBiosData,' + \
                            'SmbiosMajorVersion,SmbiosMinorVersion,Used20CallingMethod\n'
                rawBuffer += 'MININT-MHQ01OM,TRUE,0,SMBiosData,%d,{' % tableSize
                for i in range(32, len(rawData)):
                    if i != 32:
                        rawBuffer += ';'
                    rawBuffer += '%d' % ord(rawData[i])
                rawBuffer += '},%d,%d,FALSE\n' % (ord(majorVersion), ord(minorVersion))
            else:
                aCmd = 'wmic /namespace:\\\\root\\wmi path MSSmbios_RawSMBiosTables get * /format:csv'
                anInput = os.popen(aCmd, 'r')
                rawBuffer = ''
                while True:
                    rawIn = anInput.read()
                    if not rawIn:
                        break
                    rawBuffer += rawIn
                exitStatus = anInput.close()
                if exitStatus is None:
                    self.exitCode = 0
                else:
                    self.exitCode = exitStatus
        firstChar = rawBuffer[0:1]  # check for file in unicode
        if ord(firstChar) > 127:
            aBuffer = ''
            for i in range(len(rawBuffer)):
                if i < 2:
                    continue
                if not i&1:
                    aBuffer += rawBuffer[i]
            rawBuffer = aBuffer
        lines = rawBuffer.splitlines()
        names = lines[1].split(',')
        parts = lines[2].split(',')
        sysSize = 0
        for indx in range(len(names)):
            if names[indx].lower() == 'smbiosmajorversion':
                majorVersion = parts[indx]
            if names[indx].lower() == 'smbiosminorversion':
                minorVersion = parts[indx]
            if names[indx].lower() == 'size':
                sysSize = int(parts[indx])
            if names[indx].lower() == 'smbiosdata':
                exec("dataArray = %s" % \
                        (parts[indx].replace(';', ',').replace('{', '[').replace('}', ']')))
        self.smbiosVersion = "%s.%s" % (majorVersion, minorVersion)
        self.smbiosData = dataArray
        self.smbiosLength = len(self.smbiosData)
        if self.smbiosLength != sysSize:
            exp = "Size mismatch table has %d, measured %d" % (sysSize, self.smbiosLength)
            print exp
            quit()
        self.doIndexTableSearch()

    def getExitCode(self):
        return self.exitCode

    def doIndexTableSearch(self):
        curIndex = 0
        self.offsetTableList = []
        while curIndex < self.smbiosLength:
            tableIndex = curIndex
            tableNumber = self.smbiosData[curIndex]
            curIndex += self.smbiosData[curIndex + 1]
            while curIndex < self.smbiosLength:
                if self.smbiosData[curIndex] == 0:
                    if self.smbiosData[curIndex + 1] == 0:
                        curIndex += 2
                        break
                curIndex += 1
            self.offsetTableList.append([tableIndex, tableNumber])

    def getSMBiosVersion(self):
        if self.refObj:
            return self.refObj.getSMBiosVersion()
        return self.smbiosVersion

    def getDataByte(self, indx):
        if self.refObj:
            return self.refObj.getDataByte(indx)
        if indx < self.smbiosLength:
            return self.smbiosData[indx]
        else:
            raise 'index out of range'

    def getTableIndex(self, tableNumber):
        if self.refObj:
            return self.refObj.getTableIndex(tableNumber)
        if self.tableNumberGen == None or self.tableNumberGen != tableNumber:
            self.tableNumberGen = tableNumber
            self.indx_offsetTableList = 0
        while self.indx_offsetTableList < len(self.offsetTableList):
            if self.offsetTableList[self.indx_offsetTableList][1] == tableNumber:
                rtnValue = self.offsetTableList[self.indx_offsetTableList][0]
                self.indx_offsetTableList += 1
                return rtnValue
            self.indx_offsetTableList += 1
        self.tableNumberGen = None
        return None

    def getTaggedStringFromTableIndex(self, tableIndex, stringNumber):
        stringTag = "%d[" % stringNumber
        rtnString = ''
        firstString = True
        stringIndex = tableIndex + self.smbiosData[tableIndex + 1]
        while self.smbiosData[stringIndex]:
            matchTag = True
            for indx in range(len(stringTag)):
                aChar = chr(self.smbiosData[stringIndex + indx])
                if stringTag[indx] != aChar:
                    matchTag = False
                    break
            theString = ''
            while self.smbiosData[stringIndex]:
                if matchTag:
                    theString += chr(self.smbiosData[stringIndex])
                stringIndex += 1
            if matchTag:
                if not firstString:
                    rtnString += ','
                else:
                    firstString = False
                rtnString += theString
            stringIndex += 1
        return rtnString

    def getStringFromTableIndex(self, tableIndex, stringNumber):
        if self.refObj:
            return self.refObj.getStringFromTableIndex(tableIndex, stringNumber)
        if self.smbiosData[tableIndex] == 11:  # OEM strings table is different
            return self.getTaggedStringFromTableIndex(tableIndex, stringNumber)
        theStringNumber = self.smbiosData[tableIndex + stringNumber]
        if theStringNumber == 0:
            return ''
        stringsIndex = tableIndex + self.smbiosData[tableIndex + 1]
        curStringNumber = 1
        while curStringNumber < theStringNumber:
            rtnString = ''
            while self.smbiosData[stringsIndex]:
                rtnString += chr(self.smbiosData[stringsIndex])
                stringsIndex += 1
            stringsIndex += 1
            curStringNumber += 1
        rtnString = ''
        while self.smbiosData[stringsIndex]:
            rtnString += chr(self.smbiosData[stringsIndex])
            stringsIndex += 1
        return rtnString.strip()

    def getByteFromTableIndex(self, tableIndex, byteOffset):
        if self.refObj:
            return self.refObj.getByteFromTableIndex(tableIndex, byteOffset)
        if byteOffset > self.smbiosData[tableIndex + 1]:
            return ''
        return '%02X' % (self.smbiosData[tableIndex + byteOffset])

    def getWordFromTableIndex(self, tableIndex, byteOffset):
        if self.refObj:
            return self.refObj.getWordFromTableIndex(tableIndex, byteOffset)
        if byteOffset > self.smbiosData[tableIndex + 1]:
            return ''
        return '%04X' % ((self.smbiosData[tableIndex + byteOffset + 1] * 256) + \
                self.smbiosData[tableIndex + byteOffset])

    def getDWordFromTableIndex(self, tableIndex, byteOffset):
        if self.refObj:
            return self.refObj.getDWordFromTableIndex(tableIndex, byteOffset)
        if byteOffset > self.smbiosData[tableIndex + 1]:
            return ''
        aRtn = 0
        for i in (3,2,1,0):
            aRtn *= 256
            aRtn += self.smbiosData[tableIndex + byteOffset + i]
        return '%08X' % aRtn









# ################################### #
# SMBIOS Access
# interface for SMBIOS Raw
# ################################### #
class SMBiosAccess:

    def __init__(self, filename=None):
        global SMBiosAccessObj
        if SMBiosAccessObj:
            self.refObj = SMBiosAccessObj
            return
        SMBiosAccessObj = self
        self.refObj = None
        self.filename = filename
        self.rawDataObj = SMBiosRawData(self.filename)

    def getSMBiosVersion(self):
        if self.refObj:
            return self.refObj.getSMBiosVersion()
        return self.rawDataObj.getSMBiosVersion()

    def getTableIndex(self, tableNumber):
        if self.refObj:
            return self.refObj.getTableIndex(tableNumber)
        return self.rawDataObj.getTableIndex(tableNumber)

    def getStringFromTableIndex(self, tableIndex, stringNumber):
        if self.refObj:
            return self.refObj.getStringFromTableIndex(tableIndex, stringNumber)
        return self.rawDataObj.getStringFromTableIndex(tableIndex, stringNumber)

    def getByteFromTableIndex(self, tableIndex, stringNumber):
        if self.refObj:
            return self.refObj.getByteFromTableIndex(tableIndex, stringNumber)
        return self.rawDataObj.getByteFromTableIndex(tableIndex, stringNumber)

    def getWordFromTableIndex(self, tableIndex, stringNumber):
        if self.refObj:
            return self.refObj.getWordFromTableIndex(tableIndex, stringNumber)
        return self.rawDataObj.getWordFromTableIndex(tableIndex, stringNumber)

    def getDWordFromTableIndex(self, tableIndex, stringNumber):
        if self.refObj:
            return self.refObj.getDWordFromTableIndex(tableIndex, stringNumber)
        return self.rawDataObj.getDWordFromTableIndex(tableIndex, stringNumber)

    def getExitCode(self):
        if self.refObj:
            return self.refObj.getExitCode()
        return self.rawDataObj.getExitCode()









# ################################### #
# Collect Data
# Collecting the smbios details
# ################################### #
class CollectData:
    SourceSpec = 'SourceSpec'
    SMBIOSDMIType = 'SMBIOSDMIType'
    properties = 'properties'
    SMBIOSOffsetName = 'SMBIOSOffsetName'
    Offset = 'Offset'
    Length = 'Length'
    ValueType = 'ValueType'
    DefaultValue = 'DefaultValue'
    Value = 'Value'

    def __init__(self, collectJSONfilename, smbiosDumpFilename=None, outputJSONfilename=None, skipUpdate=False):
        global CollectDataObj
        if CollectDataObj:
            self.refObj = self
            CollectDataObj = self
            return
        self.refObj = None
        self.collectJSONfilename = collectJSONfilename
        self.smbiosDumpFilename = smbiosDumpFilename
        self.outputJSONfilename = outputJSONfilename
        self.skipUpdate = skipUpdate

        self.oSA = SMBiosAccess(self.smbiosDumpFilename)
        self.doParseJSONFile()

    def getExitCode(self):
        return self.oSA.getExitCode()

    def doParseJSONFile(self):
        anInput = open(self.collectJSONfilename, 'r')
        if not anInput:
            raise "Can't open %s" % self.collectJSONfilename
        self.jObj = json.load(anInput, object_pairs_hook=OrderedDict)
        self.processDict(self.jObj)
        if self.outputJSONfilename:
            with open(self.outputJSONfilename, 'w') as f:
                json.dump(self.jObj, f, indent=1)
        anInput.close()

    def processDict(self, anObj):
        if self.refObj:
            return self.refObj.processDict(anObj)
        for indx in anObj.keys():
            if isinstance(anObj[indx], dict):
                thekeys = anObj[indx].keys()
                if CollectData.SourceSpec in thekeys:
                    if anObj[indx][CollectData.SourceSpec] == 'DMTF':
                        self.add2SMBIOSDataCollect(anObj, indx)
                        continue
                else:
                    self.processDict(anObj[indx])
            else:
                if isinstance(anObj[indx], list):
                    for anItem in anObj[indx]:
                        self.processDict(anItem)

    def add2SMBIOSDataCollect(self, anObj, indx):
        tableNumber = [int(s) for s in anObj[indx][CollectData.SMBIOSDMIType].split() if s.isdigit()][0]
        if not self.skipUpdate:
            tableCount = 0
            while True:
                tableIndex = self.oSA.getTableIndex(tableNumber)
                if not tableIndex:
                    break
                tableCount += 1
                if tableCount > 1:
                    newIndex = '%s%d' % (indx, tableCount)
                    anObj[newIndex] = copy.deepcopy(anObj[indx])
        else:
            tableCount = 1
        for i in range(tableCount):
            if i > 0:
                topIndex = '%s%d' % (indx, i+1)
            else:
                topIndex = indx
            props = anObj[topIndex][CollectData.properties]
            tableIndex = self.oSA.getTableIndex(tableNumber)
            for i in range(len(props)):
                if '_comment' in props[i]:
                    continue
                aString = props[i][CollectData.Offset].lower().replace('0x', '').replace('h', '')
                aNumber = 0
                for aChar in aString:
                    if aChar >= 'a':
                        aValue = 10 + ord(aChar) - ord('a')
                    else:
                        aValue = ord(aChar) - ord('0')
                    aNumber *= 16
                    aNumber += aValue
                anOffset = aNumber
                aLngth = props[i][CollectData.Length]
                aValtyp = props[i][CollectData.ValueType]
                aField = ''
                if tableNumber == 11 and aLngth == '':
                    aField = self.oSA.getStringFromTableIndex(tableIndex, int(props[i][CollectData.SMBIOSOffsetName]))
                else:
                    if aLngth == 'BYTE' and aValtyp == 'UTF8_String':
                        aField = self.oSA.getStringFromTableIndex(tableIndex, anOffset)
                    else:
                        if aLngth == 'BYTE':
                            aField = self.oSA.getByteFromTableIndex(tableIndex, anOffset)
                        else:
                            if aLngth == 'WORD':
                                aField = self.oSA.getWordFromTableIndex(tableIndex, anOffset)
                            else:
                                if aLngth == 'DWORD':
                                    aField = self.oSA.getDWordFromTableIndex(tableIndex, anOffset)
                if not self.skipUpdate:
                    if len(aField) < 1:
                        if CollectData.DefaultValue in props[i]:
                            aField = props[i][CollectData.DefaultValue]
                    props[i][CollectData.Value] = aField








# ################################### #
# json parser for reading output from Collect Data
# ################################### #
class JSON_PARSER(object):
    def __init__(self, filename="", properties=[]):
        self.filename = filename
        self.properties = properties
        return

    def getTitle(self):
        Title = ''
        if os.path.exists(self.filename):
            fileSchema = open(self.filename, "r")
            data = json.load(fileSchema, object_pairs_hook=OrderedDict)
            if 'subjectalternativename' not in self.filename:
                component = data['PlatformConfiguration']['ComponentDetails']
                Title = component[0].keys()[0]
            else:
                component = data['SubjectAltName']['SystemInformation']
                Title = 'SubjectAltName'
        return Title

    def parser(self):
        data = OrderedDict()
        dict_1 = OrderedDict()
        dict_2 = OrderedDict()
        component = []
        final_list = []
        Title = ''
        if os.path.exists(self.filename):
            f = open(self.filename, "r")
        else:
            return final_list
        data = json.load(f, object_pairs_hook=OrderedDict)
        f.close()
        if len(data) == 0:
            return final_list
        if 'subjectalternativename' not in self.filename:
            component = data['PlatformConfiguration']['ComponentDetails']
            keys_1 = component[0].keys()
            Title = keys_1[0]
            for i in range(len(keys_1)):
                if type(component[0][keys_1[i]]) is OrderedDict:
                    keys_2 = component[0][keys_1[i]].keys()
                    dict_1 = component[0][keys_1[i]]
                    for j in range(len(keys_2)):
                        dict_2 = dict_1[keys_2[j]]
                        if type(dict_2) is list:
                            final_list.append(dict_2)
        else:
            component = data['SubjectAltName']['SystemInformation']
            keys_1 = component.keys()
            Title = 'SubjectAltName'
            for i in range(len(keys_1)):
                if type(component[keys_1[i]]) is list:
                    final_list.append(component[keys_1[i]])
        return final_list, Title








# ################################### #
# Generic SMBIOS unit
# ################################### #
class SMBIOSBaseUnit(object):
    def __init__(self, inputSchema, unitDesc):
        self.output_schema = ''
        self.input_schema = inputSchema
        self.properties = []
        self.smbios_offset_name = ''
        self.Value = ''
        self.dict2 = OrderedDict()
        self.dict1 = ()
        self.final_dict = {}
        self.keys_1 = []
        self.title = ''
        self.description = unitDesc
        self.logFileHandle = None
        self.exitStatus = 1

    def __del__(self):
        if len(self.output_schema) > 0 and os.path.exists(self.output_schema):
            os.remove(self.output_schema)
        else:
            self.log_message("ouput schema doesnot exist")

        if self.logFileHandle is not None:
            self.logFileHandle.close()


    def log_message(self, msg):
        if self.logFileHandle is not None:
            self.logFileHandle.write(msg)
            self.logFileHandle.write("\n")

    def getExitStatus(self):
        self.log_message("exitStatus is=" + str(self.exitStatus))
        return self.exitStatus

    def logInit(self, logFileName = None, logFilePath = None):
        logFile = None
        self.logFileHandle = None
        if logFilePath is not None and logFileName is not None:
            try:
                logFile = os.path.join(logFilePath, logFileName)
                self.logFileHandle = open(logFile, "w+")
                self.log_message("log file for unit:" + self.description)
            except:
                self.logFileHandle = None
                logFile = None

    def collectSMBIOSData(self):
        self.output_schema = self.input_schema + '_output'
        if os.path.exists(self.input_schema):
            collect_handle = CollectData(self.input_schema, "", self.output_schema, "")
            self.exitStatus = collect_handle.getExitCode()
        else:
            self.log_message("Input schema doesnot exist")

        json_parser_handle = JSON_PARSER(self.output_schema)
        self.properties, self.title = json_parser_handle.parser()
        self.log_message("TITLE=" + str(self.title))

    def extractUnits(self):
        if len(self.properties) != 0:
            for i in range(len(self.properties)):
                for x in range(len(self.properties[i])):
                    self.keys_1 = self.properties[i][x].keys()
                    if 'Alias' in self.keys_1:
                        self.smbios_offset_name = self.properties[i][x]['Alias']
                    if 'Value' in self.keys_1:
                        self.Value = self.properties[i][x]['Value']
                    if(len(self.Value)) == 0:
                        self.Value = 'Unknown'
                    self.dict2[self.smbios_offset_name] = self.Value
                self.dict1 = self.dict1 + (self.dict2,)
                self.log_message("Unit=" + str(i) + "  Elements captured=" + str(len(self.dict2)))
                self.dict2 = OrderedDict()
        else:
            self.log_message("No properties captured")

    def formatUnitsForOutput(self):
        self.final_dict[self.title] = self.dict1
        self.log_message("FINAL DICT SIZE=" + str(len(self.dict1)))

    def formatData(self, formatPrint):
        if formatPrint == "TXT":
            dict3 = {}
            for dict3 in self.dict1:
                print '\n'
                keys3 = dict3.keys()
                for i in range(len(keys3)):
                    print keys3[i] + "=" + dict3[keys3[i]]
        elif formatPrint == "JSON":
            print (json.dumps(self.final_dict))
            self.log_message("json dump done")
        else:
            self.log_message("Unknown format to print")

    def sequenceDefaultOperations(self, logFileName, logFilePath, formatPrint="JSON"):
        self.logInit(logFileName, logFilePath)
        self.collectSMBIOSData()
        self.extractUnits()
        self.formatUnitsForOutput()
        self.formatData(formatPrint)
        return self.getExitStatus()









# ################################### #
# Baseboard unit
# ################################### #
class BaseBoard(SMBIOSBaseUnit):
    baseboardSerialKeyAlias = 'Serial'
    def __init__(self, inputSchema, logFilePath=None):
        super(BaseBoard, self).__init__(inputSchema, "Baseboard unit")
        self.logFilePath = logFilePath
        self.type1Offset7 = ""

    def extractServiceTagFromSerial(self, serialNumber):
        serviceTag = None
        if serialNumber is not None and len(serialNumber) > 0:
            delimiter = serialNumber[0]
            if serialNumber.count(delimiter) >= 2:
                serviceTag = serialNumber.split(delimiter)[1]
        return serviceTag


    # Extract only service Tag & Piece Part ID as serial number
    # Refer Dell SMBIOS specifications
    def formatType2SerialNumber(self, serialNumber):
        formattedSerial = None
        if serialNumber is not None and len(serialNumber) > 0:
            delimiter = serialNumber[0]
            if serialNumber.count(delimiter) == 4:
                formatList = serialNumber.split(delimiter)
                del formatList[3:]
                formatList.append("")
                formattedSerial = delimiter.join(formatList)
            else:
                formattedSerial = serialNumber
        return formattedSerial


    def compareServiceTagWithType1(self, type2ServiceTag):
        if type2ServiceTag is not None and len(type2ServiceTag) > 0:
            if len(self.type1Offset7) == 0:
                smbiosAccess = SMBiosAccess()
                type1TableIndex = smbiosAccess.getTableIndex(1)
                self.type1Offset7 = smbiosAccess.getStringFromTableIndex(type1TableIndex, 7)
                type1ServiceTag = self.type1Offset7
            else:
                type1ServiceTag = self.type1Offset7
            self.log_message("Type1 svc tag is=" + type1ServiceTag)
            self.log_message("Type2 svc tag is=" + type2ServiceTag)
            return type2ServiceTag == type1ServiceTag
        else:
            return False

    # method override for baseboard
    # defines data formatting for the primary board
    def formatUnitsForOutput(self):
        if len(self.dict1) > 1:
            # mutiple baseboards present in the enclosure
            self.log_message("Multiple baseboards present in the structure")
            primaryIdentified = False
            for baseboardUnit in self.dict1:
                if self.baseboardSerialKeyAlias in baseboardUnit.keys():
                    baseboardSVCtag = self.extractServiceTagFromSerial(baseboardUnit[self.baseboardSerialKeyAlias])
                    if baseboardSVCtag is not None and self.compareServiceTagWithType1(baseboardSVCtag):
                        # primary baseboard information
                        self.log_message("Primary identified with serial number=[" + baseboardUnit[self.baseboardSerialKeyAlias] + "]")
                        baseboardUnit[self.baseboardSerialKeyAlias] = self.formatType2SerialNumber(baseboardUnit[self.baseboardSerialKeyAlias])
                        self.final_dict[self.title] = tuple((baseboardUnit,))
                        primaryIdentified = True
                        break

            if not primaryIdentified:
                self.log_message("Unable to identify the primary baseboard")
                self.final_dict[self.title] = self.dict1
        else:
            # single baseboard
            self.log_message("Single baseboard unit present in the structure")
            self.final_dict[self.title] = self.dict1

        self.log_message("FINAL DICT SIZE=" + str(len(self.dict1)))


    def dataRead(self, formatPrint="JSON"):
        self.logInit("baseboardLogFile.log", self.logFilePath)
        self.collectSMBIOSData()
        self.extractUnits()
        # specific formatting defined for baseboard
        self.formatUnitsForOutput()
        self.formatData(formatPrint)
        return self.getExitStatus()





# ################################### #
# Processor unit
# ################################### #
class Processor(SMBIOSBaseUnit):
    def __init__(self, inputSchema, logFilePath=None):
        super(Processor, self).__init__(inputSchema, "Processor unit")
        self.logFilePath = logFilePath

    def dataRead(self, formatPrint):
        self.sequenceDefaultOperations("processorLogFile.log", self.logFilePath, formatPrint)





# ################################### #
# Memory unit
# ################################### #
class Memory(SMBIOSBaseUnit):
    def __init__(self, inputSchema, logFilePath=None):
        super(Memory, self).__init__(inputSchema, "Memory unit")
        self.logFilePath = logFilePath

    def dataRead(self, formatPrint):
        self.sequenceDefaultOperations("memoryLogFile.log", self.logFilePath, formatPrint)





# ################################### #
# OEM unit
# ################################### #
class OEM(SMBIOSBaseUnit):
    def __init__(self, inputSchema, logFilePath=None):
        super(OEM, self).__init__(inputSchema, "OEM unit")
        self.logFilePath = logFilePath

    def dataRead(self, formatPrint):
        self.sequenceDefaultOperations("oemLogFile.log", self.logFilePath, formatPrint)





# ################################### #
# System Information unit
# ################################### #
class SubjectAltName(SMBIOSBaseUnit):
    def __init__(self, inputSchema, logFilePath=None):
        super(SubjectAltName, self).__init__(inputSchema, "subjectAltName unit")
        self.logFilePath = logFilePath

    def dataRead(self, formatPrint):
        self.sequenceDefaultOperations("subjectAltNameLogFile.log", self.logFilePath, formatPrint)





# ################################### #
# refer help for details to run the script
# ################################### #
if __name__ == '__main__':
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option('-i', '--inputschema', \
            action='store', type='string', default=None, dest='inputschemafile', \
            help='input schema describing details for the utility.')
    parser.add_option('-d', '--debugfilepath', \
            action='store', type='string', default=None, dest='debugFilePath', \
            help='Path to the directory to capture the log file.')
    parser.add_option('-t', '--formattype', \
            action='store', type='string', default="JSON", dest='formatType', \
            help='TXT/JSON output formatting.')
    (options, args)=parser.parse_args()

    if not options.inputschemafile:
        parser.print_help()
    else:
        inputParser = JSON_PARSER(options.inputschemafile)

        unitObject = None
        unitInputSchemaName = options.inputschemafile
        unitDebugFilePath = options.debugFilePath
        unitFormatType = options.formatType
        unitName = inputParser.getTitle()

        if unitName == 'Baseboard':
            baseboardObject = BaseBoard(unitInputSchemaName, unitDebugFilePath)
            unitObject = baseboardObject
        elif unitName == 'Processor':
            processorObject = Processor(unitInputSchemaName, unitDebugFilePath)
            unitObject = processorObject
        elif unitName == 'Memory':
            memoryObject = Memory(unitInputSchemaName, unitDebugFilePath)
            unitObject = memoryObject
        elif unitName == 'OEM':
            oemObject = OEM(unitInputSchemaName, unitDebugFilePath)
            unitObject = oemObject
        elif unitName == 'SubjectAltName':
            altNameObject = SubjectAltName(unitInputSchemaName, unitDebugFilePath)
            unitObject = altNameObject
        else:
            print "Unknown Component!"
            exit(1)

        unitObject.dataRead(unitFormatType)
        sys.exit(unitObject.getExitStatus())









