# The information in this publication is provided "as is." Dell Inc. makes no representations or warranties of any kind with respect to the information in this publication, and specifically disclaims implied warranties of merchantability or fitness for a particular purpose.
# Use, copying, and distribution of any software described in this publication requires an applicable software license.
# Copyright (c) 2020 - v1.0 Dell Inc. or its subsidiaries. All Rights Reserved. Dell Technologies, Dell, EMC, Dell EMC and other trademarks are trademarks of Dell Inc. or its subsidiaries. Other trademarks may be trademarks of their respective owners.
# standard imports
import subprocess
import json
import sys
from collections import OrderedDict
import os

UNKNOWN_CONST = "Unknown"
result = UNKNOWN_CONST
hdd_fqdd_result = UNKNOWN_CONST
idrac_hdd_cmd = "racadm storage get pdisks"
idrac_hdd_serial = UNKNOWN_CONST
idrac_hdd_size = UNKNOWN_CONST
idrac_hdd_model = UNKNOWN_CONST
idrac_hdd_fqdd = UNKNOWN_CONST
idrac_hdd_vendor = UNKNOWN_CONST
idrac_hdd_partnum = UNKNOWN_CONST
idrac_hdd_mediatype = UNKNOWN_CONST
hdd_list = []
dict2 = OrderedDict()
dict1 = ()
final_dict = {}
fields = ['Manufacturer', 'Model', 'Serial', 'FQDD', 'Media Type', 'Part Number', 'Size']
default_dict1 = ()
default_dict = OrderedDict()
default_dict['Manufacturer'] = UNKNOWN_CONST
default_dict['Model'] = UNKNOWN_CONST
default_dict['Serial'] = UNKNOWN_CONST
default_dict['FQDD'] = UNKNOWN_CONST
default_dict['Media Type'] = UNKNOWN_CONST
default_dict['Part Number'] = UNKNOWN_CONST
default_dict['Size'] = UNKNOWN_CONST
# --------------------- RETURN VALES -------------------
status = 0
SCV_HDD_CMD_FAILED = 101
SCV_HDD_FQDD_CMD_FAILED = 102
SCV_LOG = "SCV_LOG:"


def runstandalone(cmd):
    """ Runs a bash command.
    Parameters:
        cmd (str): The command to run.
    Returns:
        stdout (str):      Standard output stream of the process.
        stderr (str):      Standard error stream of the process.
        return_code (int): Return code.
    """
    # execute command
    pipe = subprocess.Popen(cmd,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stdin=subprocess.PIPE,
                            stderr=subprocess.PIPE)

    (stdout, stderr) = pipe.communicate()

    stdout = stdout.strip()
    return_code = pipe.returncode

    return stdout, return_code


def log_message(logfilehandle, msg):
    if logfilehandle is not None:
        logfilehandle.write(msg)


if (len(sys.argv) >= 3) and sys.argv[1] == "-i":
    logFile = None
    logFileHandle = None
    if len(sys.argv) >= 5 and sys.argv[3] == "-d":
        try:
            logFile = os.path.join(sys.argv[4], "harddriveLogFile.log")
            logFileHandle = open(logFile, "w+")
            logFileHandle.write(SCV_LOG + "log file for harddrives unit\n")
        except:
            logFileHandle = None
            logFile = None
    # ----------------------------------------------------------------------------------
    # getting Number Of HDDs
    # ----------------------------------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_hdd_cmd + "\n")
    result, ret_val = runstandalone(idrac_hdd_cmd)
    if(len(result)) == 0:
        status = SCV_HDD_CMD_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_hdd_cmd + " returned Empty Respone\n")
    if ret_val == 0:
        for line in result.splitlines():
            if 'Disk' in line:
                hdd_list.append(line)
    else:
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_hdd_cmd + " execution failed\n")
    for hdd_fqdd in hdd_list:
        idrac_hdd_vendor = UNKNOWN_CONST
        idrac_hdd_model = UNKNOWN_CONST
        idrac_hdd_serial = UNKNOWN_CONST
        idrac_hdd_fqdd = UNKNOWN_CONST
        idrac_hdd_mediatype = UNKNOWN_CONST
        idrac_hdd_partnum = UNKNOWN_CONST
        idrac_hdd_size = UNKNOWN_CONST

        log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_hdd_cmd + ":" + hdd_fqdd + "\n")
        hdd_fqdd_result, ret_val = runstandalone(idrac_hdd_cmd + ":" + hdd_fqdd)
        idrac_hdd_fqdd = hdd_fqdd.strip(" ")
        if len(hdd_fqdd_result) == 0:
            log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_hdd_cmd + ":" + hdd_fqdd + " returned Empty Respone\n")
            status = SCV_HDD_FQDD_CMD_FAILED
            break
        if ret_val != 0:
            log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_hdd_cmd + ":" + hdd_fqdd + " execution failed\n")
            status = SCV_HDD_FQDD_CMD_FAILED
            break
        for hdd_line in hdd_fqdd_result.splitlines():
            log_message(logFileHandle, SCV_LOG + "LINE=" + hdd_line + "\n")
            if "Manufacturer" in hdd_line:
                idrac_hdd_vendor = hdd_line.split("=")[1].strip(" ")
            if "ProductId" in hdd_line:
                idrac_hdd_model = hdd_line.split("=")[1].strip(" ")
            if "Size" in hdd_line and "BlockSizeInBytes" not in hdd_line:
                idrac_hdd_size = hdd_line.split("=")[1]
            if "SerialNumber" in hdd_line:
                idrac_hdd_serial = hdd_line.split("=")[1].strip(" ")
            if "PartNumber" in hdd_line:
                idrac_hdd_partnum = hdd_line.split("=")[1].strip(" ")
            if "MediaType" in hdd_line:
                idrac_hdd_mediatype = hdd_line.split("=")[1].strip(" ")
        i = 0
        while i < len(fields):
            dict2[fields[i]] = idrac_hdd_vendor.strip(" ")
            dict2[fields[i + 1]] = idrac_hdd_model.strip(" ")
            dict2[fields[i + 2]] = idrac_hdd_serial.strip(" ")
            dict2[fields[i + 3]] = idrac_hdd_fqdd.strip(" ")
            dict2[fields[i + 4]] = idrac_hdd_mediatype.strip(" ")
            dict2[fields[i + 5]] = idrac_hdd_partnum.strip(" ")
            dict2[fields[i + 6]] = idrac_hdd_size.strip(" ")
            i = i + 7
        dict1 = dict1 + (dict2,)
        dict2 = OrderedDict()
    if(len(sys.argv) > 1) and sys.argv[1] == "-t" and sys.argv[2] == "txt":
        dict3 = {}
        for dict3 in dict1:
            print "Manufacturer=", dict3['Manufacturer']
            print "Model=", dict3['Model']
            print "Serial=", dict3['Serial']
            print "FQDD=", dict3['FQDD']
            print "Media Type=", dict3['Media Type']
            print "Part Number=", dict3['Part Number']
            print "Size=", dict3['Size']
            print ("\n")
    else:
        if(len(dict1)) == 0:
            log_message(logFileHandle, SCV_LOG + "System have zero hard disks printing all unknow\n")
            default_dict = default_dict1 + (default_dict,)
            final_dict["HardDrive"] = default_dict
        else:
            log_message(logFileHandle, SCV_LOG + "Printing JSON output\n")
            final_dict["HardDrive"] = dict1
        print json.dumps(final_dict)
    if logFileHandle is not None:
        logFileHandle.close()
sys.exit(status)
