# The information in this publication is provided "as is." Dell Inc. makes no representations or warranties of any kind with respect to the information in this publication, and specifically disclaims implied warranties of merchantability or fitness for a particular purpose.
# Use, copying, and distribution of any software described in this publication requires an applicable software license.
# Copyright (c) 2020 - v1.0 Dell Inc. or its subsidiaries. All Rights Reserved. Dell Technologies, Dell, EMC, Dell EMC and other trademarks are trademarks of Dell Inc. or its subsidiaries. Other trademarks may be trademarks of their respective owners.
# standard imports
import subprocess
import json
import sys
import os
from collections import OrderedDict

racadmPath = "racadm "
idrac_model_cmd = racadmPath + " get idrac.info"
idrac_model = "Unknown"
idrac_version = "Unknown"
idrac_svctag_cmd = racadmPath + " getsvctag"
idrac_svctag = "Unknown"
idrac_macaddr_cmd = racadmPath + " get idrac.nic.macaddress"
idrac_macaddr = "Unknown"
idrac_manufacturer = "Unknown"
FI_CERT_FILE = "SCV_FI_CERT.pem"
idrac_cert_cmd = racadmPath + " sslcertdownload -t 5 -f " + FI_CERT_FILE
idrac_cert_data = "Unknown"
i = 0
dict1 = ()
dict2 = OrderedDict()
final_dict = {}
fields = ['Manufacturer', 'Model', 'Serial', 'Version', 'MAC Address', 'iDRAC Identity Cert']
# ----------- RETURN VALUES ---------------------
status = 0
SCV_MODEL_VERSION_CMC_FAILED = 101
SCV_SVC_TAG_FAILED = 102
SCV_MAC_CMD_FAILED = 103
SCV_FI_CERT_CMD_FAILED = 104
SCV_LOG = "SCV_LOG:"


def runstandalone(cmd):
    """ Runs a bash command.
    Parameters:
        cmd (str): The command to run.
    Returns:
        stdout (str):      Standard output stream of the process.
        stderr (str):      Standard error stream of the process.
        return_code (int): Return code.
    """
    # execute command
    pipe = subprocess.Popen(cmd,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stdin=subprocess.PIPE,
                            stderr=subprocess.PIPE)

    (stdout, stderr) = pipe.communicate()

    stdout = stdout.strip()
    return_code = pipe.returncode

    return stdout, return_code


def log_message(logfilehandle, msg):
    if logfilehandle is not None:
        logfilehandle.write(msg)


if (len(sys.argv) >= 3) and sys.argv[1] == "-i":
    logFile = None
    logFileHandle = None
    if len(sys.argv) >= 5 and sys.argv[3] == "-d":
        try:
            logFile = os.path.join(sys.argv[4], "idracLogFile.log")
            logFileHandle = open(logFile, "w+")
            logFileHandle.write(SCV_LOG + "log file for idrac unit\n")
        except:
            logFileHandle = None
            logFile = None

    # --------------------------------------------------------
    # getting iDRAC Model
    # --------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_model_cmd + "\n")
    result, ret_val = runstandalone(idrac_model_cmd)
    if len(result) == 0:
        status = SCV_MODEL_VERSION_CMC_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_model_cmd + "returned Empty Respone\n")
    if ret_val == 0:
        for line in result.splitlines():
            log_message(logFileHandle, SCV_LOG + "LINE=" + line + "\n")
            if 'Type' in line:
                idrac_model = line.split("=")[1].strip(" ")
    log_message(logFileHandle, SCV_LOG + "Type read successfully\n")
    # --------------------------------------------------------
    # getting iDRAC Service Tag
    # --------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_svctag_cmd + "\n")
    result, ret_val = runstandalone(idrac_svctag_cmd)
    if len(result) == 0:
        status = SCV_SVC_TAG_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_svctag_cmd + " returned Empty Respone\n")
    if ret_val == 0 and 'The service tag is not currently populated' not in result:
        idrac_svctag = result
        log_message(logFileHandle, SCV_LOG + "Serial number read successfully from iDRAC\n")
    else:
        log_message(logFileHandle, SCV_LOG + "SVC tag is not populated on this machine\n")
    # --------------------------------------------------------
    # getting iDRAC MAC Address
    # --------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_macaddr_cmd + "\n")
    result, ret_val = runstandalone(idrac_macaddr_cmd)
    if len(result) == 0:
        status = SCV_MAC_CMD_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_macaddr_cmd + " returned Empty Respone\n")
    if ret_val == 0:
        for line in result.splitlines():
            log_message(logFileHandle, SCV_LOG + "LINE=" + line + "\n")
            if 'MACAddress' in line:
                idrac_macaddr = line.split("=")[1].strip(" ")
    else:
        log_message(logFileHandle, SCV_LOG + "MAC address attribute read failed\n")
    # ---------------------------------------------------------
    # getting Factory certificate data
    # --------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_cert_cmd + "\n")
    result, ret_val = runstandalone(idrac_cert_cmd)
    if len(result) == 0:
        status = SCV_MAC_CMD_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_cert_cmd + " returned Empty Respone\n")
    if ret_val == 0 and 'successfully' in result and os.path.exists(FI_CERT_FILE):
        cert_file = open(FI_CERT_FILE, "r")
        idrac_cert_data = cert_file.read()
        idrac_cert_data = idrac_cert_data.split("-----BEGIN CERTIFICATE-----")[1].split("-----END CERTIFICATE-----")[0].replace('\n', '')
        log_message(logFileHandle, SCV_LOG + "Factory identity certificate extracted\n")
        cert_file.close()
    else:
        log_message(logFileHandle, SCV_LOG + "Factory identity certificate read failed\n")
    i = 0
    while i < len(fields):
        dict2[fields[i]] = idrac_manufacturer.strip(" ")
        dict2[fields[i + 1]] = idrac_model.strip(" ")
        dict2[fields[i + 2]] = idrac_svctag.strip(" ")
        dict2[fields[i + 3]] = idrac_version.strip(" ")
        dict2[fields[i + 4]] = idrac_macaddr.strip(" ")
        dict2[fields[i + 5]] = idrac_cert_data
        i = i + 6
    dict1 = dict1 + (dict2,)
    final_dict["iDRAC"] = dict1
    if(len(sys.argv) > 1) and sys.argv[1] == "-t" and sys.argv[2] == "txt":
        print "Manufacturer= ", idrac_manufacturer.strip(" ")
        print "Model=", idrac_model.strip(" ")
        print "Serial Number=", idrac_svctag
        print "MAC Address=", idrac_macaddr
        print "iDRAC Identity Cert=", idrac_cert_data
        print "Version=", idrac_version
        print "\n"
    else:
        print json.dumps(final_dict)
        log_message(logFileHandle, SCV_LOG + "JSON output printed\n")
        print "\n"
    if logFileHandle is not None:
        logFileHandle.close()
if os.path.exists(FI_CERT_FILE):
    os.remove(FI_CERT_FILE)
sys.exit(status)
