# The information in this publication is provided "as is." Dell Inc. makes no representations or warranties of any kind with respect to the information in this publication, and specifically disclaims implied warranties of merchantability or fitness for a particular purpose.
# Use, copying, and distribution of any software described in this publication requires an applicable software license.
# Copyright (c) 2020 - v1.0 Dell Inc. or its subsidiaries. All Rights Reserved. Dell Technologies, Dell, EMC, Dell EMC and other trademarks are trademarks of Dell Inc. or its subsidiaries. Other trademarks may be trademarks of their respective owners.
# standard imports
import subprocess
import json
import sys
from collections import OrderedDict
import os

UNKNOWN_CONST = "Unknown"
result = UNKNOWN_CONST
nic_fqdd_result = UNKNOWN_CONST
idrac_nic_cmd = "racadm hwinventory nic"
idrac_nic_macaddr = UNKNOWN_CONST
idrac_nic_model = UNKNOWN_CONST
idrac_nic_fqdd = UNKNOWN_CONST
idrac_nic_fqdd_cmd = "racadm hwinventory"
idrac_nic_vendor = UNKNOWN_CONST
nic_list = []
dict2 = OrderedDict()
dict1 = ()
final_dict = {}
fields = ['Manufacturer', 'Model', 'MAC Address', 'Location']
default_dict1 = ()
default_dict = OrderedDict()

default_dict['Manufacturer'] = UNKNOWN_CONST
default_dict['Model'] = UNKNOWN_CONST
default_dict['MAC Address'] = UNKNOWN_CONST
default_dict['Location'] = UNKNOWN_CONST

# -------------------- RETURN VALES ------------------------
status = 0
SCV_NW_CMD_FAILED = 101
SCV_NW_FQDD_CMD_FAILED = 102
SCV_LOG = "SCV_LOG:"


def runstandalone(cmd):
    """ Runs a bash command.
    Parameters:
        cmd (str): The command to run.
    Returns:
        stdout (str):      Standard output stream of the process.
        stderr (str):      Standard error stream of the process.
        return_code (int): Return code.
    """
    # execute command
    pipe = subprocess.Popen(cmd,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stdin=subprocess.PIPE,
                            stderr=subprocess.PIPE)

    (stdout, stderr) = pipe.communicate()

    stdout = stdout.strip()
    return_code = pipe.returncode

    return stdout, return_code


def log_message(logfilehandle, msg):
    """ logs a message to log file.
        Parameters:
            logfilehandle : handle to the log file.
            msg           : Message to be logged.
    """
    if logfilehandle is not None:
        logfilehandle.write(msg)


if (len(sys.argv) >= 3) and sys.argv[1] == "-i":
    logFile = None
    logFileHandle = None
    if len(sys.argv) >= 5 and sys.argv[3] == "-d":
        try:
            logFile = os.path.join(sys.argv[4], "networkLogFile.log")
            logFileHandle = open(logFile, "w+")
            logFileHandle.write(SCV_LOG + "log file for network unit\n")
        except:
            logFileHandle = None
            logFile = None

    # ----------------------------------------------------------------------------------
    # getting Number Of NICs
    # ----------------------------------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_nic_cmd + "\n")
    result, ret_val = runstandalone(idrac_nic_cmd)
    if len(result) == 0:
        status = SCV_NW_CMD_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_nic_cmd + " returned Empty Respone\n")
    for line in result.splitlines():
        if 'NIC' in line:
            log_message(logFileHandle, SCV_LOG + "NIC_FQDD:" + line + "\n")
            if ":" in line and "ERROR:" not in line:
                idrac_nic_fqdd = line.split(":")[0]
                nic_list.append(idrac_nic_fqdd)
    if len(nic_list) != 0:
        for nic_fqdd in nic_list:
            idrac_nic_macaddr = UNKNOWN_CONST
            idrac_nic_vendor = UNKNOWN_CONST
            idrac_nic_model = UNKNOWN_CONST

            log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_nic_fqdd_cmd + " " + nic_fqdd + "\n")
            nic_fqdd_result, ret_val = runstandalone(idrac_nic_fqdd_cmd + " " + nic_fqdd)
            if len(nic_fqdd_result) == 0:
                status = SCV_NW_FQDD_CMD_FAILED
                log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_nic_fqdd_cmd + " " + nic_fqdd + " returned Empty Respone\n")
                break
            for nic_line in nic_fqdd_result.splitlines():
                if "Permanent MAC Address" in nic_line:
                    idrac_nic_macaddr = nic_line.split("Permanent MAC Address:")[1].strip(" ")
                if "Vendor Name" in nic_line:
                    idrac_nic_vendor = nic_line.split(":")[1].strip(" ")
                if "ProductName" in nic_line:
                    idrac_nic_model = nic_line.split("-")[0].split("ProductName:")[1].strip(" ")
            i = 0
            while i < len(fields):
                dict2[fields[i]] = idrac_nic_vendor.strip(" ")
                dict2[fields[i + 1]] = idrac_nic_model.strip(" ")
                dict2[fields[i + 2]] = idrac_nic_macaddr.strip(" ")
                dict2[fields[i + 3]] = nic_fqdd
                i = i + 4
                dict1 = dict1 + (dict2,)
                dict2 = OrderedDict()
    if(len(sys.argv) > 1) and sys.argv[1] == "-t" and sys.argv[2] == "txt":
        dict3 = {}
        for dict3 in dict1:
            print "Manufacturer=", dict3['Manufacturer']
            print "Model=", dict3['Model']
            print "MAC Address=", dict3['MAC Address']
            print "NIC Partition=", dict3['Location']
            print ("\n")
    else:
        if(len(dict1)) == 0:
            log_message(logFileHandle, SCV_LOG + "System have zero network cards printing all unknow\n")
            default_dict = default_dict1 + (default_dict,)
            final_dict["Network"] = default_dict
        else:
            log_message(logFileHandle, SCV_LOG + "Printing JSON format\n")
            final_dict["Network"] = dict1
        print json.dumps(final_dict)
        print "\n"
    if logFileHandle is not None:
        logFileHandle.close()
sys.exit(status)
