# The information in this publication is provided "as is." Dell Inc. makes no representations or warranties of any kind with respect to the information in this publication, and specifically disclaims implied warranties of merchantability or fitness for a particular purpose.
# Use, copying, and distribution of any software described in this publication requires an applicable software license.
# Copyright (c) 2020 - v1.0 Dell Inc. or its subsidiaries. All Rights Reserved. Dell Technologies, Dell, EMC, Dell EMC and other trademarks are trademarks of Dell Inc. or its subsidiaries. Other trademarks may be trademarks of their respective owners.
# standard imports
import subprocess
import json
import sys
from collections import OrderedDict
import os

result = "Unknown"
idrac_psu_cmd = "racadm hwinventory"
idrac_psu_model = "Unknown"
idrac_psu_serial = "Unknown"
idrac_psu_vendor = "Unknown"
idrac_psu_location = "Unknown"
psu_list = []
dict2 = OrderedDict()
dict1 = ()
final_dict = {}
fields = ['Manufacturer', 'Model', 'Serial', 'Location']
default_dict1 = ()
default_dict = OrderedDict()
default_dict['Manufacturer'] = 'Unknown'
default_dict['Model'] = 'Unknown'
default_dict['Serial'] = 'Unknown'
default_dict['Location'] = 'Unknown'
is_this_psu = 0

# -------------------- RETURN VALES ------------------------
status = 0
SCV_NW_CMD_FAILED = 101
SCV_NW_FQDD_CMD_FAILED = 102
SCV_LOG = "SCV_LOG:"


def runstandalone(cmd):
    """ Runs a bash command.
    Parameters:
        cmd (str): The command to run.
    Returns:
        stdout (str):      Standard output stream of the process.
        stderr (str):      Standard error stream of the process.
        return_code (int): Return code.
    """
    # execute command
    pipe = subprocess.Popen(cmd,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stdin=subprocess.PIPE,
                            stderr=subprocess.PIPE)

    (stdout, stderr) = pipe.communicate()
    stdout = stdout.strip()
    return_code = pipe.returncode
    return stdout, return_code


def log_message(logfilehandle, msg):
    """ Runs a bash command.
        Parameters:
            logfilehandle : handle to the log file.
             msg           : Message to be logged.
    """
    if logfilehandle is not None:
        logfilehandle.write(msg)
        logfilehandle.write("\n")


if (len(sys.argv) >= 3) and sys.argv[1] == "-i":
    logFile = None
    logFileHandle = None
    if len(sys.argv) >= 5 and sys.argv[3] == "-d":
        try:
            logFile = os.path.join(sys.argv[4], "powersupplyLogFile.log")
            logFileHandle = open(logFile, "w+")
            logFileHandle.write(SCV_LOG + "log file for powersupply unit\n")
        except:
            logFileHandle = None
            logFile = None

    # ----------------------------------------------------------------------------------
    # getting PSU Details
    # ----------------------------------------------------------------------------------
    log_message(logFileHandle, SCV_LOG + "Executing Local racadm command " + idrac_psu_cmd)
    result, ret_val = runstandalone(idrac_psu_cmd)
    if ret_val != 0:
        status = SCV_NW_CMD_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_psu_cmd + " failed with ret code=" + str(ret_val))
    elif len(result) == 0:
        status = SCV_NW_CMD_FAILED
        log_message(logFileHandle, SCV_LOG + "Local racadm command " + idrac_psu_cmd + " returned Empty Respone")
    for psu_line in result.splitlines():
        if 'PSU' in psu_line and '[InstanceID:' in psu_line and is_this_psu == 0:
            log_message(logFileHandle, SCV_LOG + "PSU Instance=" + psu_line)
            idrac_psu_vendor = 'Unknown'
            idrac_psu_model = 'Unknown'
            idrac_psu_serial = 'Unknown'
            idrac_psu_location = 'Unknown'
            is_this_psu = 1
        if "Manufacturer" in psu_line and is_this_psu == 1:
            idrac_psu_vendor = psu_line.split("=")[1].strip(" ")
        if "Model" in psu_line and is_this_psu == 1:
            idrac_psu_model = psu_line.split("=")[1].strip(" ")
        if "SerialNumber" in psu_line and is_this_psu == 1:
            idrac_psu_serial = psu_line.split("=")[1].strip(" ")
        if "FQDD" in psu_line and is_this_psu == 1:
            idrac_psu_location = psu_line.split("=")[1].strip(" ")
        if is_this_psu == 1 and '--------------------------------------' in psu_line:
            dict2[fields[0]] = idrac_psu_vendor.strip(" ")
            dict2[fields[1]] = idrac_psu_model.strip(" ")
            dict2[fields[2]] = idrac_psu_serial.strip(" ")
            dict2[fields[3]] = idrac_psu_location.strip(" ")
            dict1 = dict1 + (dict2,)
            dict2 = OrderedDict()
            is_this_psu = 0

    log_message(logFileHandle, SCV_LOG + "PSU SEEN=" + str(len(dict1)) )
    if (len(sys.argv) > 1) and sys.argv[1] == "-t" and sys.argv[2] == "txt":
        dict3 = {}
        for dict3 in dict1:
            print "Manufacturer=", dict3['Manufacturer']
            print "Model=", dict3['Model']
            print "Serial=", dict3['Serial']
            print "Location=", dict3['Location']
            print ("\n")
    else:
        if(len(dict1)) == 0:
            log_message(logFileHandle, SCV_LOG + "System have zero psu/error cards printing all unknowns")
            default_dict = default_dict1 + (default_dict,)
            final_dict["PowerSupply"] = default_dict
        else:
            log_message(logFileHandle, SCV_LOG + "Printing JSON format with all/some values")
            final_dict["PowerSupply"] = dict1
        print json.dumps(final_dict)
        print "\n"
        log_message(logFileHandle, SCV_LOG + "Json dump done, py status=" + str(status))
    if logFileHandle is not None:
        logFileHandle.close()
sys.exit(status)
