# The information in this publication is provided "as is." Dell Inc. makes no representations or warranties of any kind with respect to the information in this publication, and specifically disclaims implied warranties of merchantability or fitness for a particular purpose.
# Use, copying, and distribution of any software described in this publication requires an applicable software license.
# Copyright (c) 2020 - v1.0 Dell Inc. or its subsidiaries. All Rights Reserved. Dell Technologies, Dell, EMC, Dell EMC and other trademarks are trademarks of Dell Inc. or its subsidiaries. Other trademarks may be trademarks of their respective owners.
# standard imports
import subprocess
import json
import sys
import os
import platform
from collections import OrderedDict

TPM2_AUTH_SETTINGS = "-a 0x40000001"  # Add auth parameters as set for your TPM. i.e. -P 2a2b2c
TPM2_EK_NV_INDEX = "0x1c00002"
distCmd = 'yum'
TPM2_TOOLS_VER_2 = '0'
TPM2_TOOLS_VER_3 = '0'
TPM_CERT_FILE = '1.cer'
abrmdActive = ""
modeCmd = "-T device"
index_found = 0
cert_size = 0
ReadCmd = ""
cert_extract_result = ""
cert_serial_result = ""
cert_PKAlgorithm = ""

EK_cert_serial = "Unknown"
EK_cert_vendor = "Unknown"
EK_cert_model = "Unknown"
EK_cert_version = "Unknown"
EK_PKAlgorithm = "Unknown"
i = 0
dict1 = ()
dict2 = OrderedDict()
final_dict = {}
vendor_oid = '2.23.133.2.1'
model_oid = '2.23.133.2.2'
verion_oid = '2.23.133.2.3'
EK_CERT_DATA = []
WINPE_EK_READ_CMD = 'TPM20_Inventory -RSAEKCERTREAD -out=1'
TPM_BIOS_RACADM_CMD = 'racadm get bios.syssecurity'
Is_Tpm_2_0 = 0
Is_Tpm_Present = 0
Is_Tpm_Enabled = 0
TPM_INFO_DICT = {}
TPM_INFO_FINAL = {}
fields = ['Manufacturer', 'Model', 'Serial', 'Version', 'Public Key Algorithm']
# ---------------- RETURN CODES ---------------------------------
status = 0
SCV_LOG = "SCV_LOG:"


def runstandalone(cmd):
    pipe = subprocess.Popen(cmd,
                            shell=True,
                            stdout=subprocess.PIPE,
                            stdin=subprocess.PIPE,
                            stderr=subprocess.PIPE)

    (stdout, stderr) = pipe.communicate()
    stdout = stdout.strip()
    return_code = pipe.returncode
    return stdout.decode('utf_8'), return_code


def log_message(logfilehandle, msg):
    if logfilehandle is not None:
        logfilehandle.write(msg)


if (len(sys.argv) >= 3) and sys.argv[1] == "-i":
    logFile = None
    logFileHandle = None
    if len(sys.argv) >= 5 and sys.argv[3] == "-d":
        try:
            logFile = os.path.join(sys.argv[4], "TPMLogFile.log")
            logFileHandle = open(logFile, "w+")
            logFileHandle.write(SCV_LOG+"log file for idrac unit\n")
        except:
            logFileHandle = None
            logFile = None

# ------------------------ TPM BIOS COMMANDS -----------------------------------
log_message(logFileHandle, SCV_LOG+"Determining whether TPM is ready or not\n")
result, ret_val = runstandalone(TPM_BIOS_RACADM_CMD)
for line in (result.splitlines()):
    if 'TpmInfo' in line:
        log_message(logFileHandle, SCV_LOG+line+"\n")
        if "No TPM present" not in line.split('=')[1]:
            Is_Tpm_Present = 1
        if '1.2' not in line.split('=')[1]:
            Is_Tpm_2_0 = 1
    if 'TpmSecurity' in line:
        log_message(logFileHandle, SCV_LOG+line+"\n")
        if 'On' in line.split('=')[1]:
            Is_Tpm_Enabled = 1
TPM_INFO_DICT['Tpm Present'] = Is_Tpm_Present
TPM_INFO_DICT['Tpm Enable'] = Is_Tpm_Enabled
TPM_INFO_FINAL['tpm info'] = TPM_INFO_DICT
log_message(logFileHandle, (json.dumps(TPM_INFO_FINAL)))
if Is_Tpm_Present == 1 and Is_Tpm_2_0 == 1 and Is_Tpm_Enabled == 1:
    # ------------------------ TPM BIOS COMMANDS END -----------------------------------
    log_message(logFileHandle, SCV_LOG+"Determinig the OS Type\n")
    if platform.system() == 'Linux':
        # ------------------X LINUX SPECIFIC CODE START X-----------------------------------#
        log_message(logFileHandle, SCV_LOG+"This is Linux OS\n")
        log_message(logFileHandle, SCV_LOG+"determine Linux varient\n")
        result, ret_val = runstandalone("cat /etc/os-release")
        if 'CentOS' in (result.splitlines()[0]):
            distCmd = 'yum'
        log_message(logFileHandle, SCV_LOG+"check if TPM present or not\n")
        result, ret_val = runstandalone("dmesg | grep -i tpm")
        log_message(logFileHandle, SCV_LOG+result+"\n")
        if((len(result) != 0) and ('No TPM chip found' not in result)):
            log_message(logFileHandle, SCV_LOG+"TPM chip is present at server proceed\n")
            log_message(logFileHandle, SCV_LOG+"Now find tpm2-tools version installed on the server\n")
            TPM2_TOOLS_VER_2, ret_val = runstandalone(distCmd + ' list installed tpm2-tools 2> /dev/null | grep --quiet -E "[ \t]+2\." && echo "1" || echo "0"')
            TPM2_TOOLS_VER_3, ret_val = runstandalone(distCmd + ' list installed tpm2-tools 2> /dev/null | grep --quiet -E "[ \t]+[3-9]+\." && echo "1" || echo "0"')
            if TPM2_TOOLS_VER_2 == '1':
                print ("TBD")
            if TPM2_TOOLS_VER_3 == '1':
                log_message(logFileHandle, SCV_LOG+"Detected TPM Tools version checking for abrmd\n")
                abrmdActive, ret_val = runstandalone('ps -aux | grep "tpm2-abrmd" | grep -v "grep"')
                if (len(abrmdActive) > 0):
                    modeCmd = "-T abrmd"
                log_message(logFileHandle, SCV_LOG+"List all NV devices and find the certficate size at 0x1c00002\n")
                nvdevices, ret_val = runstandalone("tpm2_nvlist" + " " + modeCmd)
                for line in nvdevices.splitlines():
                    if '0x1c00002' in line:
                        index_found = 1
                    if(index_found == 1):
                        if 'size:' in line:
                            cert_size = line.split('size:')[1]
                            index_found = 0
                log_message(logFileHandle, SCV_LOG+"Now form a read command to read cert_size from index\n")
                ReadCmd = "tpm2_nvread " + modeCmd + " " + TPM2_AUTH_SETTINGS + " -x" + " " + TPM2_EK_NV_INDEX + ">" + TPM_CERT_FILE
        log_message(logFileHandle, SCV_LOG+"Now run nv read command\n")
        runstandalone(ReadCmd)
        if os.path.exists(TPM_CERT_FILE):
            log_message(logFileHandle, SCV_LOG+"Certficate dumpped to "+TPM_CERT_FILE+"\n")
            cert_serial_result, ret_val = runstandalone("openssl x509 -inform der -in "+TPM_CERT_FILE+" -noout -serial")
            log_message(logFileHandle, SCV_LOG+"Serial:"+cert_serial_result+"\n")
            cert_extract_result, ret_val = runstandalone("openssl x509 -inform der -in "+TPM_CERT_FILE+" -text -noout | grep -A1 'X509v3 Subject Alternative Name'|tail -n1")
            log_message(logFileHandle, SCV_LOG+"Other Data:"+cert_serial_result+"\n")
            cert_PKAlgorithm, ret_val = runstandalone("openssl x509 -inform der -in "+TPM_CERT_FILE+" -text -noout | grep -i 'Public Key Algorithm'")
            log_message(logFileHandle, SCV_LOG + "[PKA:" + cert_PKAlgorithm + "] ret=[" + str(ret_val) + "]\n")
        # ------------------X LINUX SPECIFIC CODE END X-----------------------------------#
    else:
        # ------------------X WINDOWS SPECIFIC CODE START X-----------------------------------#
        log_message(logFileHandle, SCV_LOG+"This is WINDOWS OS\n")
        result, ret_val = runstandalone(WINPE_EK_READ_CMD)
        log_message(logFileHandle, SCV_LOG+result+"\n")
        if os.path.exists(TPM_CERT_FILE):
            log_message(logFileHandle, SCV_LOG+"Certficate dumpped to "+TPM_CERT_FILE+"\n")
            cert_serial_result, ret_val = runstandalone("openssl x509 -inform der -in "+TPM_CERT_FILE+" -noout -serial")
            log_message(logFileHandle, SCV_LOG+"Serial:"+cert_serial_result+"\n")
            cert_extract_result_temp, ret_val = runstandalone("openssl x509 -inform der -in "+TPM_CERT_FILE+" -text -noout ")
            log_message(logFileHandle, SCV_LOG+"Other Data Temp:"+cert_extract_result_temp+"\n")
            for line in cert_extract_result_temp.splitlines():
                if 'DirName' in line:
                    cert_extract_result = line
                elif 'Public Key Algorithm' in line:
                    cert_PKAlgorithm = line
            log_message(logFileHandle, SCV_LOG+"Other Data:"+cert_extract_result+"\n")
            log_message(logFileHandle, SCV_LOG+"cert_PKAlgorithm="+cert_PKAlgorithm+"\n")
        # ------------------X WINDOWS SPECIFIC CODE END X-----------------------------------#
if(len(cert_serial_result) != 0) and (len(cert_extract_result) != 0):
    EK_CERT_DATA = cert_extract_result.split('/')
    for item in range(len(EK_CERT_DATA)):
        if vendor_oid in EK_CERT_DATA[item]:
            EK_cert_vendor = EK_CERT_DATA[item].split('=')[1]
        if model_oid in EK_CERT_DATA[item]:
            EK_cert_model = EK_CERT_DATA[item].split('=')[1]
        if verion_oid in EK_CERT_DATA[item]:
            EK_cert_version = EK_CERT_DATA[item].split('=')[1]
    EK_cert_serial = cert_serial_result.split('=')[1]
    EK_PKAlgorithm = cert_PKAlgorithm.split(':')[1]
i = 0
while i < len(fields):
    dict2[fields[i]] = EK_cert_vendor.strip(" ")
    dict2[fields[i+1]] = EK_cert_model.strip(" ")
    dict2[fields[i+2]] = EK_cert_serial.strip(" ")
    dict2[fields[i+3]] = EK_cert_version.strip(" ")
    dict2[fields[i+4]] = EK_PKAlgorithm.strip("  ")
    i = i+5
dict1 = dict1+(dict2,)
final_dict["TPM"] = dict1
if(len(sys.argv) == 3) and sys.argv[1] == "-t" and sys.argv[2] == "txt":
    print ("\n")
    print ("Manufacturer=", EK_cert_vendor.strip(" "))
    print ("Model=", EK_cert_model.strip(" "))
    print ("Serial=", EK_cert_serial.strip(" "))
    print ("Version=", EK_cert_version.strip(" "))
    print ("Public Key Algorithm=", EK_PKAlgorithm.strip(" "))
else:
    print (json.dumps(final_dict))
    print ("\n")
if os.path.exists(TPM_CERT_FILE):
    os.remove(TPM_CERT_FILE)
sys.exit(status)
